;;; ska-local-keys.el --- my own major mode dependend key-bindings
;; Copyright (C) 2000-2002 Stefan Kamphausen

;; Author: Stefan Kamphausen <mail@skamphausen.de>
;; Time-stamp: <17-Feb-2004 11:57:59 ska>

;; Keywords: 
;; This file is not part of XEmacs.

;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program; see the file COPYING. If not, write to the Free
;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
;; 02111-1307, USA.


;;; Commentary:
;; In this file we define defuns for loading in each mode specific
;; hook. Each defun follows the naming convention
;; ska-<major-mode-name>-keys
;; Note that skeleton template key-bindings will be loaded from this
;; file, too.

;;; To use:
;;  1. (load "ska-local-keys" t nil)
;;           reads this file
;;  2. For each mode you need to call the appropriate defun in the
;;     hook, e.g:
;; (add-hook 'c-mode-hook
;;        '(lambda ()
;;           ;; my keybindings
;;           (ska-c-mode-keys c-mode-map)
;;           ))

;;; Code:
;; My own prefix for globally available commands: C-b
;; (just like C-x and C-c for xemacs' defaults)
(global-unset-key '[(control b)])

;; Newsgroup says it would be best to use a different modifier (Hyper,
;; Super) as personal prefix.

;; Maybe this is better:?
;(define-prefix-command 'user-mode-specific-prefix t)
;(defvar user-mode-specific-map (symbol-function
;'user-mode-specific-prefix) "\
;The default keymap for all mode specific actions defined by the
;user.")
;(define-key global-map '[(control b)] 'user-mode-specific-prefix)

;;{{{ Programming Keys For All Modes
;;----------------------------------
(defun ska-coding-keys (map)
  "Sets the key bindings which shall be available in all programming
languages. Argument MAP is the local keymap (e.g. cperl-mode-map)."
  (define-key map '[(return)]                 'newline-and-indent)
  (define-key map '[(control b) (d)]          'chb-insert-debug-output)
  (define-key map '[(control b) (\;)]         'chb-comment-region-or-line)
  (define-key map '[(control b) (\:)]         'chb-uncomment-region-or-line)
  (define-key map '[(control b) (c)]          'chb-copy-and-comment-region-or-line)
  )
;;}}}

;;{{{ C and C++ Mode
;;------------------
(ska-safe-require 'ska-skel-c)
;; This defun is slightly different from the others since it works for
;; two major modes. Therefore the map to use is passed
(defun ska-c-common-mode-keys (map)
  "Set my personal keys for C and C++.
Argument MAP is c-mode-map or c++-mode-map."
  (define-key map '[(meta tab)]                       'hippie-expand)
  (define-key map '[(control b) (control a)]          'ska-edit-automake-file)
  (define-key map '[(control b) (control b)]          'compile)
  (define-key map '[(control b) (control f)]          'fume-rescan-buffer)
  (define-key map '[(control b) (control m)]          'manual-entry)
  ;; macros, templates, skeletons:
  (define-key map '[(control b) (m) (f) (f)]          'ska-skel-c-fflush)
  (define-key map '[(control b) (m) (f) (p)]          'ska-skel-c-fprintf)
  (define-key map '[(control b) (m) (i)]              'ska-skel-c-include)
  (define-key map '[(control b) (m) (m)]              'ska-skel-c-main)
  (define-key map '[(control b) (m) (c)]              'ska-skel-c-comment)
  (define-key map '[(control b) (m) (n)]              'ska-skel-c-printf-newline)
  (define-key map '[(control b) (m) (p)]              'ska-skel-c-printf)
  (define-key map '[(control b) (m) (P)]              'ska-skel-c-printf-flush)
  (define-key map '[(control b) (m) (l) (w)]          'ska-skel-c-loop-while)
  )

(defun ska-c++-mode-keys ()
  "Set my personal keys for C++ mode."
  (local-set-key '[(control b) (m) (c)]              'ska-skel-cc-class)
  (local-set-key '[(control b) return]               'ska-skel-cc-endl)
  (local-set-key '[(control b) (m) (l) (f)]          'ska-skel-cc-loop-for)
  )
(defun ska-c-mode-keys ()
  "Set my personal keys for plain C mode."
  (local-set-key '[(control b) (m) (c)]              'ska-skel-c-comment)
  (local-set-key '[(control b) return]               'ska-skel-cc-endl)
  (local-set-key '[(control b) (m) (l) (f)]          'ska-skel-c-loop-for)
  )
;;}}}

;;{{{ Perl Mode
;;-------------
(require 'ska-skel-perl)
(defun ska-cperl-mode-keys ()
  "Setting local keybindings for major mode: perl."
  (local-set-key '[(meta tab)]               'hippie-expand)
  (local-set-key '[(control b) (control b)]  'executable-interpret)
  ;; skeletons and makros MIGHT use C-b C-s as prefix if they get too many
  (local-set-key '[(control b) (control f)]  'ska-skel-perl-file-loop)
  (local-set-key '[(control b) (control s)]  'ska-skel-perl-sub)
  (local-set-key '[(control b) (control i)]  'ska-skel-perl-prog-id)
  (local-set-key '[(control b) (control o)]  'ska-skel-perl-options)
  )
;;}}}

;;{{{ Ruby Mode
;;-------------
(require 'ska-skel-ruby)
(defun ska-ruby-mode-keys ()
  "Setting local keybindings for major mode: Ruby."
  (local-set-key '[(meta tab)]               'hippie-expand)
  (local-set-key '[(control b) (control b)]  'run-ruby)
  (local-set-key '[(control b) (control e)]  'ruby-insert-end)
  (local-set-key '[(control b) (control d)]  'ska-skel-ruby-def)
  (local-set-key '[(control b) (control c)]  'ska-skel-ruby-class)
  (local-set-key '[(control b) (control h)]  'ska-skel-ruby-header)
  (local-set-key '[(control b) (control s)]  'ska-skel-ruby-string-subst)
  (local-set-key '[(control b) (a)]          'ska-skel-ruby-array-new)
  (local-set-key '[(control b) (h)]          'ska-skel-ruby-hash-new)
  (local-set-key '[(control b) (d)]          'ska-skel-ruby-do-with-args)
  )
;;}}}

;;{{{ SQL Mode
;;------------
(require 'ska-skel-sql)
(defun ska-sql-mode-keys ()
  "Setting local keybindings for major mode: SQL."
  (local-set-key '[(control b) (control v)] 'ska-skel-sql-view)
  (local-set-key '[(control b) (control v)] 'ska-skel-sql-build)
  )
;;}}}

;;{{{ LaTeX Mode
;;--------------
(defun ska-latex-mode-keys ()
  "Set keys for latex-mode (AUCTeX)."
  (local-set-key '[(tab)]                    'LaTeX-indent-line)
  (define-key LaTeX-math-keymap
    (concat LaTeX-math-abbrev-prefix "/")    'LaTeX-math-frac)
)
(defun ska-bibtex-mode-keys ()
  "Set keys for bibtex-mode."
  (local-set-key '[(control return)]         'bibtex-next-field)
)
;;}}}

;;{{{ Matlab Mode
(ska-safe-require 'ska-skel-matlab)
(defun ska-matlab-mode-keys ()
  "Set keys for matlab-mode."
  (local-set-key '[(control b) (m) (f)]   'ska-skel-matlab-function)
  )
;;}}}

;;{{{ Emacs Lisp Mode
;;-------------------
(defun ska-elisp-mode-keys ()
  "Set keys for elisp-mode."
  (local-set-key '[(control b) (control b)]  'eval-buffer)
  (local-set-key '[(control b) (control d)]  'll-debug-insert-debug-output)
    )
;;}}}

;;{{{ VM and Mail
;;------
(require 'ska-skel-mail)
(defun ska-vm-keys ()
  "Set keybindings in VM Mailreader."
  (local-set-key '[(a)]                      'vm-toolbar-autofile-message)
)
(defun ska-mail-keys ()
  "Set keybindings for mail writing buffers."
  (local-set-key '[(control b) (control r)]  'ska-skel-mail-regards)
  (local-set-key '[(control b) (control k)]  'ska-skel-mail-coffee-or-tee)
  (local-set-key '[(control b) (control b)]  'ska-mail-insert-answer-gap)
  )
;;}}}

;;{{{ Speedbar
;;------------
(defun ska-speedbar-keys ()
  "Set keybindings in major navigation tool Speedbar."
  (define-key speedbar-key-map '[(space)]    'ska-speedbar-toggle-expand)
  )
;;}}}

;;{{{ Makefile
(defun ska-makefile-keys ()
  "Set keybindings for makefile-mode."
  (local-set-key '[(control b) (control b)]   'compile)
  )
;;}}}

;;{{{ SGML, XML and HTML
;;----------------------
(defun ska-sgml-keys ()
  "Set keybindings for all modes done by PSGML."
  (local-set-key '[(control c) (control c)]  'sgml-insert-element)
  (local-set-key '[(control c) (a)]          'sgml-insert-attribute)
  (local-set-key '[(return)] 'newline-and-indent)
  )
;;}}}

;;{{{ PCL-CVS
(defun ska-cvs-keys ()
  "Set keybindings for CVS mode."
  (local-set-key '[(G)] 'cvs-update)
  (local-set-key '[(S)] 'cvs-status)
  (local-set-key '[(Q)] 'cvs-examine)
  (local-set-key '[(c)] 'cvs-mode-commit))
;;}}}

(provide 'ska-local-keys)

;;; local-keys.el ends here
;; Local variables:
;; mode: emacs-lisp
;; folded-file: t
;; end:
